@@ -1,4 +1,9 @@
-#!/usr/bin/env python
+# This application is released under the GNU General Public License 
+# v3 (or, at your option, any later version). You can find the full 
+# text of the license under http://www.gnu.org/licenses/gpl.txt. 
+# By using, editing and/or distributing this software you agree to 
+# the terms and conditions of this license. 
+# Thank you for using free software!
 
 # Options-system (c) RYX (aka Rico Pfaus) 2007 <ryx@ryxperience.com>
 #
@@ -29,6 +34,13 @@ import gtk, gobject
 import xml.dom.minidom
 from xml.dom.minidom import Node
 
+# translation stuff
+import gettext
+gettext.textdomain('screenlets')
+gettext.bindtextdomain('screenlets', screenlets.INSTALL_PREFIX +  '/share/locale')
+
+def _(s):
+	return gettext.gettext(s)
 
 # -----------------------------------------------------------------------
 # Option-classes and subclasses
@@ -122,7 +134,7 @@ class StringOption (Option):
 class IntOption (Option):
 	"""An Option for values of type number (can be int or float)."""
 	
-	def __init__ (self, group, name, default, label, desc,  min=0, max=0, 
+	def __init__ (self, group, name, default, label, desc,  min=-100000, max=100000, 
 		increment=1, **keyword_args):
 		Option.__init__(self, group, name, default, label, desc, **keyword_args)
 		self.min = min
@@ -222,16 +234,21 @@ class AccountOption (Option):
 		# THIS IS A WORKAROUND FOR A BUG IN KEYRING (usually we would use
 		# gnomekeyring.get_default_keyring_sync() here):
 		# find first available keyring
-		keyring_list = gnomekeyring.list_keyring_names_sync()
-		if len(keyring_list) == 0:
-			raise Exception('No keyrings found. Create one first!')
+		self.keyring_list = gnomekeyring.list_keyring_names_sync()
+		if len(self.keyring_list) == 0:
+			raise Exception('No keyrings found. Please create one first!')
 		else:
 			# we prefer the default keyring
-			if keyring_list.count('default') > 0:
-				self.keyring = 'default'
-			else:
-				print "Warn: No default keyring found, storage not permanent!"
-				self.keyring = keyring_list[0]
+ 			try:
+ 				self.keyring = gnomekeyring.get_default_keyring_sync()
+ 			except:
+				if "session" in self.keyring_list:
+					print "Warning: No default keyring found, using session keyring. Storage is not permanent!"
+					self.keyring = "session"
+				else:
+					print "Warning: Neither default nor session keyring found, assuming keyring %s!" % self.keyring_list[0]
+					self.keyring = self.keyring_list[0]
+
 	
 	def on_import (self, strvalue):
 		"""Import account info from a string (like 'username:auth_token'), 
@@ -240,10 +257,10 @@ class AccountOption (Option):
 		# split string into username/auth_token
 		#data = strvalue.split(':', 1)
 		(name, auth_token) = strvalue.split(':', 1)
-		if name and auth_token:
-			# read pass from storage
-			try:
-				pw = gnomekeyring.item_get_info_sync('session', 
+ 		if name and auth_token:
+ 			# read pass from storage
+ 			try:
+				pw = gnomekeyring.item_get_info_sync(self.keyring, 
 					int(auth_token)).get_secret()
 			except Exception, ex:
 				print "ERROR: Unable to read password from keyring: %s" % ex
@@ -256,13 +273,13 @@ class AccountOption (Option):
 	def on_export (self, value):
 		"""Export the given tuple/list containing a username and a password. The
 		function stores the password in the gnomekeyring and returns a
-		string in form 'username:auth_token'."""
-		# store password in storage
-		attribs = dict(name=value[0])
-		auth_token = gnomekeyring.item_create_sync('session', 
-			gnomekeyring.ITEM_GENERIC_SECRET, value[0], attribs, value[1], True) 
-		# build value from username and auth_token
-		return value[0] + ':' + str(auth_token)
+ 		string in form 'username:auth_token'."""
+ 		# store password in storage
+ 		attribs = dict(name=value[0])
+		auth_token = gnomekeyring.item_create_sync(self.keyring, 
+			gnomekeyring.ITEM_GENERIC_SECRET, value[0], attribs, value[1], True)
+ 		# build value from username and auth_token
+ 		return value[0] + ':' + str(auth_token)
 
 """#TEST:
 o = AccountOption('None', 'pop3_account', ('',''), 'Username/Password', 'Enter username/password here ...')
@@ -276,6 +293,10 @@ print o.on_import(exported_account)
 import sys
 sys.exit(0)"""
 
+class TimeOption (ColorOption):
+	"""An Option-subclass for string-values that contain dates."""
+
+
 # -----------------------------------------------------------------------
 # EditableOptions-class and needed functions
 # -----------------------------------------------------------------------
@@ -347,7 +368,7 @@ def create_option_from_node (node, groupname):
 	return None
 	
 
-class EditableOptions:
+class EditableOptions(object):
 	"""The EditableOptions can be inherited from to allow objects to export 
 	editable options for editing them with the OptionsEditor-class.
 	NOTE: This could use some improvement and is very poorly coded :) ..."""
@@ -375,7 +396,7 @@ class EditableOptions:
 		try:
 			self.__options_groups__[option.group]['options'].append(option)
 		except:
-			print "Options: Error- group " + option.group + " not defined."
+			print "Options: Error - group %s not defined." % option.group
 			return False
 		# now add the option
 		self.__options__.append(option)
@@ -399,6 +420,14 @@ class EditableOptions:
 				o.disabled = True
 				return True
 		return False
+
+	def enable_option(self, name):
+		"""Enable the inputs for a certain Option."""
+		for o in self.__options__:
+			if o.name == name:
+				o.disabled = False
+				return True
+		return False
 	
 	def export_options_as_list (self):
 		"""Returns all editable options within a list (without groups)
@@ -584,7 +613,7 @@ class ListOptionDialog (gtk.Dialog):
 		return ret
 				
 	def button_callback (self, widget, id):
-		print "PRESS: "+id
+		print "PRESS: %s" % id
 		if id == 'remove':
 			self.remove_selected_item()
 		if id == 'add':
@@ -621,7 +650,7 @@ class OptionsDialog (gtk.Dialog):
 	def __init__ (self, width, height):
 		# call gtk.Dialog.__init__
 		super(OptionsDialog, self).__init__(
-			"Edit Options", flags=gtk.DIALOG_DESTROY_WITH_PARENT |
+			_("Edit Options"), flags=gtk.DIALOG_DESTROY_WITH_PARENT |
 			gtk.DIALOG_NO_SEPARATOR,
 			buttons = (#gtk.STOCK_REVERT_TO_SAVED, gtk.RESPONSE_APPLY,
 				gtk.STOCK_CLOSE, gtk.RESPONSE_OK))
@@ -648,8 +677,6 @@ class OptionsDialog (gtk.Dialog):
 		self.create_about_page()
 		self.create_themes_page()
 		self.create_options_page()
-		# crete tooltips-object
-		self.tooltips = gtk.Tooltips()
 	
 	# "public" functions
 	
@@ -737,59 +764,69 @@ class OptionsDialog (gtk.Dialog):
 	def show_themes_for_screenlet (self, obj):
 		"""Update the Themes-page to display the available themes for the
 		given Screenlet-object."""
-		# list with found themes
-		found_themes = []
-		# now check all paths for themes
+
+
+		dircontent = []
+		screenlets.utils.refresh_available_screenlet_paths()
+
 		for path in screenlets.SCREENLETS_PATH:
 			p = path + '/' + obj.get_short_name() + '/themes'
 			print p
 			#p = '/usr/local/share/screenlets/Clock/themes'	# TEMP!!!
 			try:
-				dircontent = os.listdir(p)
+				dc = os.listdir(p)
+				for d in dc:
+					dircontent.append({'name':d, 'path':p+'/'})
 			except:
 				print "Path %s not found." % p
+
+		# list with found themes
+		found_themes = []
+			
+		# check all themes in path
+		for elem in dircontent:
+			# load themes with the same name only once
+			if found_themes.count(elem['name']):
 				continue
-			# check all themes in path
-			for name in dircontent:
-				# load themes with the same name only once
-				if found_themes.count(name):
-					continue
-				found_themes.append(name)
-				# build full path of theme.conf
-				theme_conf	= p + '/' + name + '/theme.conf'
-				# if dir contains a theme.conf
-				if os.access(theme_conf, os.F_OK):
-					# load it and create new list entry
-					ini = screenlets.utils.IniReader()
-					if ini.load(theme_conf):
-						# check for section
-						if ini.has_section('Theme'):
-							# get metainfo from theme
-							th_fullname	= ini.get_option('name', 
-								section='Theme')
-							th_info		= ini.get_option('info', 
-								section='Theme')
-							th_version	= ini.get_option('version', 
-								section='Theme')
-							th_author	= ini.get_option('author', 
-								section='Theme')
-							# create array from metainfo and add it to liststore
-							info = [name, th_fullname, th_info, th_author, 
-								th_version]
-							self.liststore.append([info])
-				else:
-					# no theme.conf in dir? just add theme-name
-					self.liststore.append([[name, '-', '-', '-', '-']])
-				# is it the active theme?
-				if name == obj.theme_name:
-					# select it in tree
-					print "active theme is: %s" % name
-					sel = self.tree.get_selection()
-					if sel:
-						it = self.liststore.get_iter_from_string(\
-							str(len(self.liststore)-1))
-						if it:
-							sel.select_iter(it)
+			found_themes.append(elem['name'])
+			# build full path of theme.conf
+			theme_conf = elem['path'] + elem['name'] + '/theme.conf'
+			# if dir contains a theme.conf
+			if os.access(theme_conf, os.F_OK):
+				# load it and create new list entry
+				ini = screenlets.utils.IniReader()
+				if ini.load(theme_conf):
+					# check for section
+					if ini.has_section('Theme'):
+						# get metainfo from theme
+						th_fullname	= ini.get_option('name', 
+							section='Theme')
+						th_info		= ini.get_option('info', 
+							section='Theme')
+						th_version	= ini.get_option('version', 
+							section='Theme')
+						th_author	= ini.get_option('author', 
+							section='Theme')
+						# create array from metainfo and add it to liststore
+						info = [elem['name'], th_fullname, th_info, th_author, 
+							th_version]
+						self.liststore.append([info])
+					else:
+						# no theme section in theme.conf just add theme-name
+						self.liststore.append([[elem['name'], '-', '-', '-', '-']])
+			else:
+				# no theme.conf in dir? just add theme-name
+				self.liststore.append([[elem['name'], '-', '-', '-', '-']])
+			# is it the active theme?
+			if elem['name'] == obj.theme_name:
+				# select it in tree
+				print "active theme is: %s" % elem['name']
+				sel = self.tree.get_selection()
+				if sel:
+					it = self.liststore.get_iter_from_string(\
+						str(len(self.liststore)-1))
+					if it:
+						sel.select_iter(it)
 
 	# UI-creation
 	
@@ -813,7 +850,7 @@ class OptionsDialog (gtk.Dialog):
 		self.page_about.pack_start(self.infotext, 1, 1, 5)
 		# add page
 		self.page_about.show()
-		self.main_notebook.append_page(self.page_about, gtk.Label('About '))	
+		self.main_notebook.append_page(self.page_about, gtk.Label(_('About ')))	
 	
 	def create_options_page (self):
 		"""Create the "Options"-tab."""
@@ -825,14 +862,14 @@ class OptionsDialog (gtk.Dialog):
 		self.page_options.add(self.vbox_editor)
 		# show/add page
 		self.page_options.show()
-		self.main_notebook.append_page(self.page_options, gtk.Label('Options '))
+		self.main_notebook.append_page(self.page_options, gtk.Label(_('Options ')))
 
 	def create_themes_page (self):
 		"""Create the "Themes"-tab."""
-		self.page_themes = gtk.VBox(spacing=3)
-		self.page_themes.set_border_width(5)
+		self.page_themes = gtk.VBox(spacing=5)
+		self.page_themes.set_border_width(10)
 		# create info-text list
-		txt = gtk.Label('Themes allow you to easily switch the appearance of your Screenlets. On this page you find a list of all available themes for this Screenlet.')
+		txt = gtk.Label(_('Themes allow you to easily switch the appearance of your Screenlets. On this page you find a list of all available themes for this Screenlet.'))
 		txt.set_size_request(450, -1)
 		txt.set_line_wrap(True)
 		txt.set_alignment(0.0, 0.0)
@@ -861,20 +898,25 @@ class OptionsDialog (gtk.Dialog):
 		# show/add page
 		self.page_themes.add(vbox)
 		self.page_themes.show()
-		self.main_notebook.append_page(self.page_themes, gtk.Label('Themes '))
+		self.main_notebook.append_page(self.page_themes, gtk.Label(_('Themes ')))
 		
 	def __render_cell(self, tvcolumn, cell, model, iter):
 		"""Callback for rendering the cells in the theme-treeview."""
 		# get attributes-list from Treemodel
 		attrib = model.get_value(iter, 0)
+
 		# set colors depending on state
 		col = '555555'
 		name_uc = attrib[0][0].upper() + attrib[0][1:]
 		# create markup depending on info
 		if attrib[1] == '-' and attrib[2] == '-':
 			mu = '<b><span weight="ultrabold" size="large">' + name_uc + \
-			'</span></b> (no info available)'
+			'</span></b> (' + _('no info available') + ')'
 		else:
+			if attrib[1] == None : attrib[1] = '-'
+			if attrib[2] == None : attrib[2] = '-'
+			if attrib[3] == None : attrib[3] = '-'
+			if attrib[4] == None : attrib[4] = '-'
 			mu = '<b><span weight="ultrabold" size="large">' + name_uc + \
 				'</span></b> v' + attrib[4] + '\n<small><span color="#555555' +\
 				'">' + attrib[2].replace('\\n', '\n') + \
@@ -949,18 +991,18 @@ class OptionsDialog (gtk.Dialog):
 			widget.set_value(value)
 			widget.connect("value-changed", self.options_callback, option)
 		elif t == ColorOption:
-			widget = gtk.ColorButton()
+			widget = gtk.ColorButton(gtk.gdk.Color(int(value[0]*65535), int(value[1]*65535), int(value[2]*65535)))
 			widget.set_use_alpha(True)
-			widget.set_alpha(int(value[3]*65535/255))
-			widget.set_color(gtk.gdk.Color(int(value[0]*65535/255), 
-				int(value[1]*65535/255), int(value[2]*65535/255)))
+#			print value
+#			print value[3]
+			widget.set_alpha(int(value[3]*65535))
 			widget.connect("color-set", self.options_callback, option)
 		elif t == FontOption:
 			widget = gtk.FontButton()
 			widget.set_font_name(value)
 			widget.connect("font-set", self.options_callback, option)		
 		elif t == FileOption:
-			widget = gtk.FileChooserButton("Choose File")
+			widget = gtk.FileChooserButton(_("Choose File"))
 			widget.set_filename(value)
 			widget.set_size_request(180, 28)
 			widget.connect("selection-changed", self.options_callback, option)
@@ -969,7 +1011,7 @@ class OptionsDialog (gtk.Dialog):
 				gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK),
 				action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
 			widget = gtk.FileChooserButton(dlg)
-			widget.set_title("Choose Directory")
+			widget.set_title(_("Choose Directory"))
 			widget.set_filename(value)
 			widget.set_size_request(180, 28)
 			widget.connect("selection-changed", self.options_callback, option)
@@ -978,7 +1020,7 @@ class OptionsDialog (gtk.Dialog):
 			entry = gtk.Entry()
 			entry.set_text(value)
 			entry.set_editable(False)
-			but = gtk.Button('')
+			but = gtk.Button()
 			# util to reload preview image
 			def create_preview (filename):
 				if filename and os.path.isfile(filename):
@@ -995,7 +1037,7 @@ class OptionsDialog (gtk.Dialog):
 			def but_callback (widget):
 				dlg = gtk.FileChooserDialog(buttons=(gtk.STOCK_CANCEL, 
 					gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
-				dlg.set_title("Choose Image")
+				dlg.set_title(_("Choose Image"))
 				dlg.set_keep_above(True)
 				dlg.set_filename(entry.get_text())
 				flt = gtk.FileFilter()
@@ -1036,8 +1078,9 @@ class OptionsDialog (gtk.Dialog):
 			but.show()
 			widget.show()
 			# add tooltips
-			#self.tooltips.set_tip(but, 'Select Image ...')
-			self.tooltips.set_tip(but, option.desc)
+			#but.set_tooltip_text(_('Select Image ...'))
+			but.set_tooltip_text(option.desc)
+			
 		elif t == ListOption:
 			entry= gtk.Entry()
 			entry.set_editable(False)
@@ -1045,7 +1088,7 @@ class OptionsDialog (gtk.Dialog):
 			entry.show()
 			img = gtk.Image()
 			img.set_from_stock(gtk.STOCK_EDIT, 1)
-			but = gtk.Button('')
+			but = gtk.Button()
 			but.set_image(img)
 			def open_listeditor(event):
 				# open dialog
@@ -1062,8 +1105,8 @@ class OptionsDialog (gtk.Dialog):
 				dlg.destroy()
 			but.show()
 			but.connect("clicked", open_listeditor)
-			self.tooltips.set_tip(but, 'Open List-Editor ...')
-			self.tooltips.set_tip(entry, option.desc)
+			but.set_tooltip_text(_('Open List-Editor ...'))
+			entry.set_tooltip_text(option.desc)
 			widget = gtk.HBox()
 			widget.add(entry)
 			widget.add(but)
@@ -1077,20 +1120,53 @@ class OptionsDialog (gtk.Dialog):
 			input_pass.set_visibility(False)	# password
 			input_pass.set_text(value[1])
 			input_pass.show()
-			but = gtk.Button('Apply', gtk.STOCK_APPLY)
+			but = gtk.Button(_('Apply'), gtk.STOCK_APPLY)
 			but.show()
 			but.connect("clicked", self.apply_options_callback, option, widget)
 			vb.add(input_name)
 			vb.add(input_pass)
 			vb.show()
-			self.tooltips.set_tip(but, 'Apply username/password ...')
-			self.tooltips.set_tip(input_name, 'Enter username here ...')
-			self.tooltips.set_tip(input_pass, 'Enter password here ...')
+			but.set_tooltip_text(_('Apply username/password ...'))
+			input_name.set_tooltip_text(_('Enter username here ...'))
+			input_pass.set_tooltip_text(_('Enter password here ...'))
 			widget.add(vb)
 			widget.add(but)
+		elif t == TimeOption:
+			widget = gtk.HBox()
+			input_hour		= gtk.SpinButton()#climb_rate=1.0)
+			input_minute	= gtk.SpinButton()
+			input_second	= gtk.SpinButton()
+			input_hour.set_range(0, 23)
+			input_hour.set_max_length(2)
+			input_hour.set_increments(1, 1)
+			input_hour.set_numeric(True)
+			input_hour.set_value(value[0])
+			input_minute.set_range(0, 59)
+			input_minute.set_max_length(2)
+			input_minute.set_increments(1, 1)
+			input_minute.set_numeric(True)
+			input_minute.set_value(value[1])
+			input_second.set_range(0, 59)
+			input_second.set_max_length(2)
+			input_second.set_increments(1, 1)
+			input_second.set_numeric(True)
+			input_second.set_value(value[2])
+			input_hour.connect('value-changed', self.options_callback, option)
+			input_minute.connect('value-changed', self.options_callback, option)
+			input_second.connect('value-changed', self.options_callback, option)
+			input_hour.set_tooltip_text(option.desc)
+			input_minute.set_tooltip_text(option.desc)
+			input_second.set_tooltip_text(option.desc)
+			widget.add(input_hour)
+			widget.add(gtk.Label(':'))
+			widget.add(input_minute)
+			widget.add(gtk.Label(':'))
+			widget.add(input_second)
+			widget.add(gtk.Label('h'))
+			widget.show_all()
 		else:
 			widget = gtk.Entry()
-			print "unsupported type '"+str(t)+"'"
+			print "unsupported type ''" % str(t)
 		hbox = gtk.HBox()
 		label = gtk.Label()
 		label.set_alignment(0.0, 0.0)
@@ -1103,11 +1179,11 @@ class OptionsDialog (gtk.Dialog):
 				widget.set_sensitive(False)
 				label.set_sensitive(False)
 			#label.set_mnemonic_widget(widget)
-			self.tooltips.set_tip(widget, option.desc)
+			widget.set_tooltip_text(option.desc)
 			widget.show()
 			# check if needs Apply-button
 			if option.realtime == False:
-				but = gtk.Button('Apply', gtk.STOCK_APPLY)
+				but = gtk.Button(_('Apply'), gtk.STOCK_APPLY)
 				but.show()
 				but.connect("clicked", self.apply_options_callback, 
 					option, widget)
@@ -1162,8 +1238,13 @@ class OptionsDialog (gtk.Dialog):
 				if c.__class__ == gtk.VBox:
 					c2 = c.get_children()
 					val = (c2[0].get_text(), c2[1].get_text())
+		elif t == TimeOption:
+			box = widget.get_parent()
+			inputs = box.get_children()
+			val = (int(inputs[0].get_value()), int(inputs[2].get_value()), 
+				int(inputs[4].get_value()))
 		else:
-			print "OptionsDialog: Unknown option type: " + str(t)
+			print "OptionsDialog: Unknown option type: %s" % str(t)
 			return None
 		# return the value
 		return val
@@ -1173,7 +1254,7 @@ class OptionsDialog (gtk.Dialog):
 	# TODO: custom callback/signal for each option?
 	def options_callback (self, widget, optionobj):
 		"""Callback for handling changed-events on entries."""
-		print "Changed: "+optionobj.name
+		print "Changed: %s" % optionobj.name
 		if self.__shown_object:
 			# if the option is not real-time updated,
 			if optionobj.realtime == False:
@@ -1225,7 +1306,25 @@ if __name__ == "__main__":
 		text_suffix	= '</b>'
 		note_text	= ""	# hidden option because val has its own editing-dialog
 		random_pin_pos	= True
-		
+		opt1 = 'testval 1'
+		opt2 = 'testval 2'
+		filename2	= ''
+		filename	= ''
+		dirname		= ''
+		font = 'Sans 12'
+		color = (0.1, 0.5, 0.9, 0.9)
+		name = 'a name'
+		name2 = 'another name'
+		combo_test = 'el2'
+		flt = 0.5
+		x = 10
+		y = 25
+		width = 30
+		height = 50
+		is_sticky = False
+		is_widget = False
+		time	= (12, 32, 49)		# a time-value (tuple with ints)
+
 		def __init__ (self):
 			EditableOptions.__init__(self)
 			# Add group
@@ -1277,6 +1376,8 @@ if __name__ == "__main__":
 				'Image-Test', 'Testing the ImageOption-type ...'))
 			self.add_option(DirectoryOption('Test', 'dirname', os.environ['HOME'],
 				'Directory-Test', 'Testing a FileOption-type ...'))
+			self.add_option(TimeOption('Test','time', self.time,
+				'TimeOption-Test', 'Testing a TimeOption-type ...'))
 			# TEST
 			self.disable_option('width')
 			self.disable_option('height')
